#include "iomanip.hpp"

static std::ostream& gray(std::ostream& out, int val, int code) {
    if (val < 0 || val > 23) {
        throw std::invalid_argument("gray value out of range");
    }
    out << "\x1b[" << code << ";5;" << 232 + val << "m";
    return out;
}

static std::ostream& rgb(std::ostream& out, int r, int g, int b, int code) {
    if (r < 0 || r > 5 || g < 0 || g > 5 || b < 0 || b > 5) {
        throw std::invalid_argument("rgb color component out of range");
    }
    out << "\x1b[" << code << ";5;" << (16 + 36 * r + 6 * g + b) << "m";
    return out;
}

std::ostream& sc::io::hide_cursor(std::ostream& out) {
    out << "\x1b[?25l";
    return out;
}

std::ostream& sc::io::show_cursor(std::ostream& out) {
    out << "\x1b[?25h";
    return out;
}

std::ostream& sc::io::reset(std::ostream& out) {
    out << "\x1b[m";
    return out;
}

std::ostream& sc::io::bold(std::ostream& out) {
    out << "\x1b[1m";
    return out;
}

std::ostream& sc::io::italic(std::ostream& out) {
    out << "\x1b[3m";
    return out;
}

std::ostream& sc::io::underline(std::ostream& out) {
    out << "\x1b[4m";
    return out;
}

std::ostream& sc::io::overline(std::ostream& out) {
    out << "\x1b[53m";
    return out;
}

std::ostream& sc::io::strikethru(std::ostream& out) {
    out << "\x1b[9m";
    return out;
}

std::ostream& sc::io::blinkslow(std::ostream& out) {
    out << "\x1b[5m";
    return out;
}

std::ostream& sc::io::blinkfast(std::ostream& out) {
    out << "\x1b[6m";
    return out;
}

std::ostream& sc::io::reverse(std::ostream& out) {
    out << "\x1b[7m";
    return out;
}

std::ostream& sc::io::blackf(std::ostream& out) {
    out << "\x1b[30m";
    return out;
}

std::ostream& sc::io::redf(std::ostream& out) {
    out << "\x1b[31m";
    return out;
}

std::ostream& sc::io::greenf(std::ostream& out) {
    out << "\x1b[32m";
    return out;
}

std::ostream& sc::io::yellowf(std::ostream& out) {
    out << "\x1b[33m";
    return out;
}

std::ostream& sc::io::bluef(std::ostream& out) {
    out << "\x1b[34m";
    return out;
}

std::ostream& sc::io::magentaf(std::ostream& out) {
    out << "\x1b[35m";
    return out;
}

std::ostream& sc::io::cyanf(std::ostream& out) {
    out << "\x1b[36m";
    return out;
}

std::ostream& sc::io::whitef(std::ostream& out) {
    out << "\x1b[37m";
    return out;
}

std::ostream& sc::io::defaultf(std::ostream& out) {
    out << "\x1b[39m";
    return out;
}

std::ostream& sc::io::blackb(std::ostream& out) {
    out << "\x1b[40m";
    return out;
}

std::ostream& sc::io::redb(std::ostream& out) {
    out << "\x1b[41m";
    return out;
}

std::ostream& sc::io::greenb(std::ostream& out) {
    out << "\x1b[42m";
    return out;
}

std::ostream& sc::io::yellowb(std::ostream& out) {
    out << "\x1b[43m";
    return out;
}

std::ostream& sc::io::blueb(std::ostream& out) {
    out << "\x1b[44m";
    return out;
}

std::ostream& sc::io::magentab(std::ostream& out) {
    out << "\x1b[45m";
    return out;
}

std::ostream& sc::io::cyanb(std::ostream& out) {
    out << "\x1b[46m";
    return out;
}

std::ostream& sc::io::whiteb(std::ostream& out) {
    out << "\x1b[47m";
    return out;
}

std::ostream& sc::io::defaultb(std::ostream& out) {
    out << "\x1b[49m";
    return out;
}

sc::io::grayf::grayf(int val) : _val {val} {}

std::ostream& sc::io::operator<<(std::ostream &out, const grayf& obj) {
    return gray(out, obj._val, 38);
}

sc::io::grayb::grayb(int val) : _val {val} {}

std::ostream& sc::io::operator<<(std::ostream &out, const grayb& obj) {
    return gray(out, obj._val, 48);
}

sc::io::rgbf::rgbf(int r, int g, int b) :
    _r {r}, _g {g}, _b {b} {}

std::ostream& sc::io::operator<<(std::ostream &out, const rgbf& obj) {
    return rgb(out, obj._r, obj._g, obj._b, 38);
}

sc::io::rgbb::rgbb(int r, int g, int b) :
    _r {r}, _g {g}, _b {b} {}

std::ostream& sc::io::operator<<(std::ostream &out, const rgbb& obj) {
    return rgb(out, obj._r, obj._g, obj._b, 48);
}
